<?php
    
    namespace artbox\catalog\catalog\models;

    use artbox\core\models\Image;
    use artbox\core\models\Language;
    use artbox\core\models\traits\AliasableTrait;
    use Yii;
    use yii\behaviors\TimestampBehavior;
    use yii\db\ActiveQuery;
    use yii\db\ActiveRecord;
    use yii\helpers\Json;
    use yii2tech\ar\position\PositionBehavior;
    use yii2tech\ar\variation\VariationBehavior;

    /**
     * This is the model class for table "brand".
     *
     * @property integer     $id
     * @property integer     $image_id
     * @property integer     $sort
     * @property boolean     $status
     * @property integer     $created_at
     * @property integer     $updated_at
     *
     * @property Image       $image
     * @property BrandLang[] $brandLangs
     * @property Language[]  $languages
     * @property Product[]   $products
     *
     
     */
    class Brand extends ActiveRecord
    {
        /**
         * @inheritdoc
         */
        use AliasableTrait;
        public static function tableName()
        {
            return 'brand';
        }
    
        /**
         * @inheritdoc
         */
        public function behaviors()
        {
            return [
                'translations' => [
                    'class'                             => VariationBehavior::className(),
                    'variationsRelation'                => 'languages',
                    'defaultVariationRelation'          => 'language',
                    'variationOptionReferenceAttribute' => 'language_id',
                    'optionModelClass'                  => Language::className(),
                    'defaultVariationOptionReference'   => function () {
                        return Language::$current->id;
                    },
                    'optionQueryFilter'                 => function (ActiveQuery $query) {
                        $query->where(
                            [
                                'status' => true,
                            ]
                        );
                    },
                    'variationAttributeDefaultValueMap' => [
                        'title' => 'name',
                    ],
                ],
                'timestamp' => [
                    'class' => TimestampBehavior::className(),
                ],
                'positionBehavior'     => [
                    'class'             => PositionBehavior::className(),
                    'positionAttribute' => 'sort',
                ],
            ];
        }
        
        /**
         * @inheritdoc
         */
        public function rules()
        {
            return [
                [
                    [
                        'image_id',
                        'sort',
                    ],
                    'integer',
                ],
                [
                    [ 'status' ],
                    'boolean',
                ],
                [
                    [ 'image_id' ],
                    'exist',
                    'skipOnError'     => true,
                    'targetClass'     => Image::className(),
                    'targetAttribute' => [ 'image_id' => 'id' ],
                ],
            ];
        }
        
    
        /**
         * @return \yii\db\ActiveQuery
         */
        public function getLanguage()
        {
            return $this->hasOne(BrandLang::className(), [ 'brand_id' => 'id' ]);
        }
        
        public function attributeLabels()
        {
            return [
                'id'         => Yii::t('catalog', 'ID'),
                'image_id'   => Yii::t('catalog', 'ID изображения'),
                'sort'       => Yii::t('catalog', 'Sort'),
                'status'     => Yii::t('catalog', 'Status'),
                'created_at' => Yii::t('catalog', 'Created At'),
                'updated_at' => Yii::t('catalog', 'Updated At'),
                'title'      => Yii::t('catalog', 'Title'),
            ];
        }
        
        /**
         * @return \yii\db\ActiveQuery
         */
        public function getImage()
        {
            return $this->hasOne(Image::className(), [ 'id' => 'image_id' ]);
        }
        
        /**
         * @return \yii\db\ActiveQuery
         */
        public function getBrandLangs()
        {
            return $this->hasMany(BrandLang::className(), [ 'brand_id' => 'id' ])
                        ->inverseOf('brand');
        }
        
        /**
         * @return \yii\db\ActiveQuery
         */
        public function getLanguages()
        {
            return $this->hasMany(BrandLang::className(), [ 'brand_id' => 'id' ]);
        }
        
        /**
         * @return \yii\db\ActiveQuery
         */
        public function getProducts()
        {
            return $this->hasMany(Product::className(), [ 'brand_id' => 'id' ])
                        ->inverseOf('brand');
        }
    
        /**
         * @return string
         */
        public function getRoute()
        {
            return Json::encode(
                [
                    'brand/view',
                    'id' => $this->id,
                ]
            );
        }
    }
