<?php
    
    namespace artbox\catalog\catalog\models;

    use artbox\catalog\filter\models\Option;
    use yii\behaviors\TimestampBehavior;
    use yii\db\ActiveRecord;
    use yii2tech\ar\position\PositionBehavior;
    use common\models\catalog\Product;

    /**
     * This is the model class for table "variant".
     *
     * @property int    $id
     * @property int    $product_id
     * @property string $sku
     * @property string $price
     * @property string $price_old
     * @property int    $stock
     * @property bool   $status
     * @property int    $sort
     * @property int    $created_at
     * @property int    $updated_at
     * @property int    $image_id
     */
    class Variant extends ActiveRecord
    {
        public $optionIds;
        
        /**
         * @inheritdoc
         */
        public static function tableName()
        {
            return 'variant';
        }
    
        public function behaviors()
        {
            return [
                'position'           => [
                    'class'             => PositionBehavior::class,
                    'positionAttribute' => 'sort',
                    'groupAttributes'   => [
                        'product_id',
                    ],
                ],
                'timestamp'          => [
                    'class' => TimestampBehavior::class,
                ],
            ];
        }
    
        /**
         * @inheritdoc
         */
        public function rules()
        {
            return [
                [
                    [
                        'optionIds',
                    ],
                    'safe',
                ],
                [
                    [
                        'product_id',
                        'sku',
                        'stock',
                    ],
                    'required',
                ],
                [
                    [
                        'product_id',
                        'stock',
                        'sort',
                        'created_at',
                        'updated_at',
                    ],
                    'integer',
                ],
                [
                    [
                        'price',
                        'price_old',
                    ],
                    'number',
                ],
                [
                    [ 'status' ],
                    'boolean',
                ],
                [
                    [ 'sku' ],
                    'string',
                    'max' => 255,
                ],
                [
                    [ 'product_id' ],
                    'exist',
                    'skipOnError'     => true,
                    'targetClass'     => Product::class,
                    'targetAttribute' => [ 'product_id' => 'id' ],
                ],
            ];
        }
    
        /**
         * @inheritdoc
         */
        public function attributeLabels()
        {
            return [
                'id'         => 'ID',
                'product_id' => 'Product ID',
                'sku'        => \Yii::t('catalog', 'Sku'),
                'price'      => \Yii::t('catalog', 'Price'),
                'price_old'  => \Yii::t('catalog', 'Price Old'),
                'stock'      => \Yii::t('catalog', 'Stock'),
                'status'     => \Yii::t('catalog', 'Status'),
                'sort'       => \Yii::t('catalog', 'Sort'),
                'created_at' => 'Created At',
                'updated_at' => 'Updated At',
                'image_id'   => 'Image ID',
            ];
        }
    
        public function getOptions()
        {
            return $this->hasMany(Option::class, [ 'id' => 'option_id' ])
                        ->viaTable('option_to_variant', [ 'variant_id' => 'id' ]);
        }
    
        public function getProduct()
        {
            return $this->hasOne(Product::class, [ 'id' => 'product_id' ]);
        }
    }
