<?php
    /**
     * @var \artbox\core\services\Languages              $languages
     * @var \artbox\catalog\catalog\models\Product       $product
     * @var \artbox\catalog\catalog\models\ProductLang[] $productLangs
     * @var \artbox\core\models\Alias[]                  $aliases
     * @var array                                        $brands
     * @var array                                        $options
     * @var \artbox\catalog\catalog\models\Variant[]     $variants
     * @var array                                        $categoriesData
     * @var array                                        $categoriesOptions
     * @var \yii\web\View                                $this
     */
    
    use artbox\catalog\assets\Select2Tree;
    use artbox\core\admin\assets\DateRangePicker;
    use artbox\core\admin\helpers\RelationInfo;
    use artbox\core\admin\widgets\ImageInput;
    use artbox\core\helpers\SlugifyDecorator;
    use artbox\core\widgets\GalleryWidget;
    use kartik\select2\Select2;
    use yii\base\InvalidConfigException;
    use yii\web\View;
    use yii\widgets\ActiveForm;
    use artbox\core\admin\widgets\Form;
    
    Select2Tree::register($this);
    
    $js = <<<JS
    $('form').on('afterValidate', function(form, attribute, messages){
        if (messages.length > 0){
        new PNotify({
                        title: 'Info',
                        text: 'Проверьте заполнение полей',
                        type: 'info',
                        styling: "bootstrap3"
                    });
        }
    });

    $("#product-categoryids").select2ToTree();
JS;
    
    $this->registerJs($js, View::POS_READY);
?>


<?php
    $form = ActiveForm::begin();
?>

<div class="x_panel x_panel-scroll tabs-lang-wr">
    
    <?php
        foreach ($productLangs as $key => $productLang) {
            $langSuffix = $languages->getActive()
            ?>
          
          <div class="style tabs-lang" style="display: <?= ( $languages->getDefault(
              )->id === $productLang->language_id ) ? 'block' : 'none' ?>;">
            <div class="style title-edit-page">Языковой блок</div>
              
              <?php $attributeField = $form->field($productLang, "[$key]title")
                                           ->label(
                                               $productLang->getAttributeLabel('title') . ' (' . $languages->getUrlById(
                                                   $productLang->language_id
                                               ) . ')'
                                           );
                  echo $attributeField;
              ?>
              
              <?= $form->field($productLang, "[$key]description")
                       ->widget(
                           \mihaildev\ckeditor\CKEditor::class,
                           [
                      
                               'editorOptions' => \yii\helpers\ArrayHelper::merge(
                                   \mihaildev\elfinder\ElFinder::ckeditorOptions(
                                       'elfinder',
                                       [
                                           'language' => 'ru',
                                       ]
                                   ),
                                   [
                                       'preset'         => 'standart',
                                       'removeButtons'  => 'SelectAll,Find,Replace,Flash,Smiley,PageBreak,ShowBlocks,Save,NewPage,Print,Templates,ImageButton,Select,Button,Textarea,Radio,Checkbox,Form,Subscript,Superscript,CopyFormatting,RemoveFormat,CreateDiv,Language,HiddenField,About,TextField,Font,FontSize',
                                       'allowedContent' => true,
                                   ]
                               ),
                           ]
                       )
                       ->label(
                           $productLang->getAttributeLabel('description') . ' (' . $languages->getUrlById(
                               $productLang->language_id
                           ) . ')'
                       ); ?>
              <?php foreach ($this->context->module->productLanguageFields as $field) {
                  switch ($field[ 'type' ]) {
                      case Form::STRING :
                          echo $form->field($productLang, $field[ 'name' ])
                                    ->textInput()
                                    ->label(
                                        $productLang->getAttributeLabel(
                                            $field[ 'name' ]
                                        ) . ' (' . $languages->getUrlById(
                                            $productLang->language_id
                                        ) . ')'
                                    );
                          
                          break;
                      
                      case Form::TEXTAREA :
                          echo $form->field($productLang, $field[ 'name' ])
                                    ->textarea()
                                    ->label(
                                        $productLang->getAttributeLabel(
                                            $field[ 'name' ]
                                        ) . ' (' . $languages->getUrlById(
                                            $productLang->language_id
                                        ) . ')'
                                    );
                          break;
                      case Form::WYSIWYG :
                          $form->field($productLang, $field[ 'name' ])
                               ->widget(
                                   \mihaildev\ckeditor\CKEditor::class,
                                   [
                                  
                                       'editorOptions' => \yii\helpers\ArrayHelper::merge(
                                           \mihaildev\elfinder\ElFinder::ckeditorOptions(
                                               'elfinder',
                                               [
                                                   'language' => 'ru',
                                               ]
                                           ),
                                           [
                                               'preset'         => 'standart',
                                               'removeButtons'  => 'SelectAll,Find,Replace,Flash,Smiley,PageBreak,ShowBlocks,Save,NewPage,Print,Templates,ImageButton,Select,Button,Textarea,Radio,Checkbox,Form,Subscript,Superscript,CopyFormatting,RemoveFormat,CreateDiv,Language,HiddenField,About,TextField,Font,FontSize',
                                               'allowedContent' => true,
                                           ]
                                       ),
                                   ]
                               )
                               ->label(
                                   $productLang->getAttributeLabel($field[ 'name' ]) . ' (' . $languages->getUrlById(
                                       $productLang->language_id
                                   ) . ')'
                               );
                          
                          break;
                      case Form::IMAGE:
                          echo $form->field(
                              $productLang,
                              $field[ 'name' ],
                              [ 'options' => [ 'class' => 'form-group filed-upload-imgs' ] ]
                          )
                                    ->widget(
                                        ImageInput::class,
                                        [
                                            'showPreview' => true,
                                        ]
                                    )
                                    ->label(
                                        $productLang->getAttributeLabel(
                                            $field[ 'name' ]
                                        ) . ' (' . $languages->getUrlById(
                                            $productLang->language_id
                                        ) . ')'
                                    );
                          break;
                  }
                  
                  ?>
              
              
              <?php } ?>
          
          
          </div>
            
            <?php
        }
    ?>

</div>

<div class="x_panel x_panel-scroll">
  <div class="wrapp-blocks-edit-page columns-2">
    
    <div class="style title-edit-page">Основной блок</div>
      
      <?= $form->field($product, 'image', [ 'options' => [ 'class' => 'form-group filed-upload-imgs' ] ])
               ->widget(
                   ImageInput::class,
                   [
                       'showPreview' => true,
                   ]
               ); ?>
    
    <div class="form-group">
      <div class="style wrapp-status-check-adm">
        <div class="style"><label for="">Маркеры товаров</label></div>
        <div style="margin-left: -10px;margin-right: -10px">
            <?= $form->field($product, 'isTop')
                     ->checkbox(
                         [
                             'class' => 'flat',
                         ]
                     ) ?>
            
            <?= $form->field($product, 'isNew')
                     ->checkbox(
                         [
                             'class' => 'flat',
                         ]
                     ) ?>
            
            <?= $form->field($product, 'isSale')
                     ->checkbox(
                         [
                             'class' => 'flat',
                         ]
                     ) ?>
        </div>
      </div>
    </div>
      
      <?= $form->field($product, 'brand_id')
               ->widget(
                   Select2::class,
                   [
                       'options'       => [
                           'placeholder' => \Yii::t('app', 'Выберите бренд'),
                       ],
                       'data'          => $brands,
                       'pluginOptions' => [
                           'allowClear' => true,
                       ],
                   ]
               ) ?>
      
      <?= $form->field($product, 'categoryIds')
               ->dropDownList(
                   $categoriesData,
                   [
                       'options'  => $categoriesOptions,
                       'multiple' => true,
                   ]
               ) ?>
      
      <?= $form->field($product, "optionIds")
               ->widget(
                   Select2::class,
                   [
                       'options'       => [
                           'placeholder' => \Yii::t('app', 'Выберите фильтры'),
                           'multiple'    => true,
                       ],
                       'data'          => $options,
                       'pluginOptions' => [
                           'allowClear'         => true,
                           'minimumInputLength' => 1,
                           'language'           => [
                               'errorLoading' => new \yii\web\JsExpression(
                                   "function () { return 'Waiting for results...'; }"
                               ),
                           ],
                           'ajax'               => [
                               'url'      => \yii\helpers\Url::to([ '/filter/group/find-options' ]),
                               'dataType' => 'json',
                               'data'     => new \yii\web\JsExpression(
                                   'function(params) {
                                    return {
                                        q:params.term
                                    };
                                 }'
                               ),
                           ],
                           'escapeMarkup'       => new \yii\web\JsExpression(
                               'function (markup) {
                                return markup;
                             }'
                           ),
                           'templateResult'     => new \yii\web\JsExpression(
                               'function (brand) {
                                return brand.text;
                             }'
                           ),
                           'templateSelection'  => new \yii\web\JsExpression(
                               'function (brand) {
                                return brand.text;
                             }'
                           ),
                       ],
                   ]
               ) ?>
      
      <?php foreach ($this->context->module->productCommonFields as $field) {
          switch ($field[ 'type' ]) {
              case Form::STRING :
                  echo $form->field($product, $field[ 'name' ])
                            ->textInput();
                  
                  break;
              
              case Form::TEXTAREA :
                  echo $form->field($product, $field[ 'name' ])
                            ->textarea();
                  break;
              case Form::WYSIWYG :
                  $form->field($product, $field[ 'name' ])
                       ->widget(
                           \mihaildev\ckeditor\CKEditor::class,
                           [
                          
                               'editorOptions' => \yii\helpers\ArrayHelper::merge(
                                   \mihaildev\elfinder\ElFinder::ckeditorOptions(
                                       'elfinder',
                                       [
                                           'language' => 'ru',
                                       ]
                                   ),
                                   [
                                       'preset'         => 'standart',
                                       'removeButtons'  => 'SelectAll,Find,Replace,Flash,Smiley,PageBreak,ShowBlocks,Save,NewPage,Print,Templates,ImageButton,Select,Button,Textarea,Radio,Checkbox,Form,Subscript,Superscript,CopyFormatting,RemoveFormat,CreateDiv,Language,HiddenField,About,TextField,Font,FontSize',
                                       'allowedContent' => true,
                                   ]
                               ),
                           ]
                       );
                  
                  break;
              case Form::IMAGE:
                  echo $form->field(
                      $product,
                      $field[ 'name' ],
                      [ 'options' => [ 'class' => 'form-group filed-upload-imgs' ] ]
                  )
                            ->widget(
                                ImageInput::class,
                                [
                                    'showPreview' => true,
                                ]
                            );
                  break;
              case Form::BOOL:
                  echo $form->field(
                      $product,
                      $field[ 'name' ],
                      [
                          'template' => "<div class='div_ style' style='margin-bottom: 10px;'>{label}</div>\n{input}\n{error}",
                      ]
                  )
                            ->checkbox(
                                [
                                    'class' => 'switchery',
                                ],
                                false
                            );
                  break;
              
              case Form::NUMBER:
                  echo $form->field(
                      $product,
                      $field[ 'name' ],
                      [
                          'template' => '{label}<span class="pos-minus-adm"></span>{input}<span class="pos-pluse-adm"></span>
                            {error}{hint}',
                          'options'  => [ 'class' => 'form-group filed-sort' ],
                      ]
                  );
                  break;
              
              case Form::RELATION:
                  $isMultiple = $field[ 'multiple' ] ?? false;
                  $relationInfo = new RelationInfo($product::className(), $field[ 'relationName' ]);
                  echo $form->field($product, $field[ 'name' ])
                            ->widget(
                                Select2::classname(),
                                [
                                    'data'          => $relationInfo->getAllFields($field[ 'relationAttribute' ]),
                                    'options'       => [
                                        'placeholder' => \Yii::t('core', 'Select ...'),
                                        'multiple'    => $isMultiple,
                                    ],
                                    'pluginOptions' => [
                                        'allowClear' => true,
                                    ],
                                ]
                            );
                  break;
              case Form::SELECT:
                  $isMultiple = $field[ 'multiple' ] ?? false;
                  echo $form->field($product, $field[ 'name' ])
                            ->widget(
                                Select2::classname(),
                                [
                                    'data'          => $field[ 'data' ],
                                    'options'       => [
                                        'placeholder' => \Yii::t('core', 'Select ...'),
                                        'multiple'    => $isMultiple,
                                    ],
                                    'pluginOptions' => [
                                        'allowClear' => true,
                                    ],
                                ]
                            );
                  
                  break;
              case Form::DATE:
                  DateRangePicker::register($this);
                  $mindate = date("d-m-Y H:i");
                  $value = $this->model->{$field[ 'name' ]};
                  $js = <<< JS
var dateRangeInputFrom = $('#date_field');
var dateFormat = '{$product->clientDateFormat}';
dateRangeInputFrom.daterangepicker({
  singleDatePicker: true,
      autoUpdateInput: true,
       showDropdowns: true,
       minDate: '{$mindate}',
       timePicker: true,
       timePicker24Hour: true,
      locale: {
         cancelLabel: 'Clear',
          format: dateFormat
      }
}, function(){
      
      });
dateRangeInputFrom.on('cancel.daterangepicker', function() {
  //do something, like clearing an input
  dateRangeInputFrom.val('');
});
if ("{$value}" == ""){
     dateRangeInputFrom.val('');
}
JS;
                  $this->registerJs($js, \yii\web\View::POS_READY);
                  echo $form->field(
                      $product,
                      $field[ 'name' ],
                      [ 'options' => [ 'class' => 'form-group date-search' ] ]
                  )
                            ->textInput([ 'id' => "date_field" ]);
                  break;
              case Form::WIDGET:
                  echo $form->field($product, $field[ 'name' ])
                            ->widget($field[ 'className' ], $field[ 'options' ] ?? []);
                  break;
              default:
                  throw new InvalidConfigException(
                      \Yii::t(
                          'core',
                          'Unavailable or unknown type: {type}',
                          [
                              'type' => $field[ 'type' ],
                          ]
                      )
                  );
          }
          
      } ?>
  </div>
</div>

<div class="x_panel x_panel-scroll tabs-lang-wr">
    
    <?php
        foreach ($aliases as $key => $alias) {
            ?>
          
          <div class="style tabs-lang" style="display: <?= ( $languages->getDefault(
              )->id === $alias->language_id ) ? 'block' : 'none' ?>;">
            <div class="style title-edit-page">СЕО блок</div>
            
            <div class="seo-btns-wr">
                <?= SlugifyDecorator::decorate(
                    $form->field($alias, "[$key]value"),
                    [ '/seo/alias/slugify' ],
                    $attributeField,
                    false,
                    $alias->language_id
                )
                                    ->textInput([ 'maxlength' => true ]); ?>
            </div>
              <?= $form->field($alias, "[$key]title")
                       ->label(
                           $alias->getAttributeLabel('title') . ' (' . $languages->getUrlById(
                               $alias->language_id
                           ) . ')'
                       ) ?>
              
              <?= $form->field($alias, "[$key]description")
                       ->textarea(
                           [
                               'rows' => 6,
                           ]
                       )
                       ->label(
                           $alias->getAttributeLabel('description') . ' (' . $languages->getUrlById(
                               $alias->language_id
                           ) . ')'
                       ) ?>
              <?= $form->field($alias, "[$key]h1")
                       ->label(
                           $alias->getAttributeLabel('h1') . ' (' . $languages->getUrlById(
                               $alias->language_id
                           ) . ')'
                       ) ?>
              
              <?= $form->field($alias, "[$key]seo_text")
                       ->widget(
                           \mihaildev\ckeditor\CKEditor::class,
                           [
                      
                               'editorOptions' => \yii\helpers\ArrayHelper::merge(
                                   \mihaildev\elfinder\ElFinder::ckeditorOptions(
                                       'elfinder',
                                       [
                                           'language' => 'ru',
                                       ]
                                   ),
                                   [
                                       'preset'         => 'standart',
                                       'removeButtons'  => 'SelectAll,Find,Replace,Flash,Smiley,PageBreak,ShowBlocks,Save,NewPage,Print,Templates,ImageButton,Select,Button,Textarea,Radio,Checkbox,Form,Subscript,Superscript,CopyFormatting,RemoveFormat,CreateDiv,Language,HiddenField,About,TextField,Font,FontSize',
                                       'allowedContent' => true,
                                   ]
                               ),
                           ]
                       ); ?>
              
              <?= $form->field($alias, "[$key]robots")
                       ->widget(
                           Select2::class,
                           [
                               'data' => [
                                   'index,follow'     => 'index,follow',
                                   'noindex,follow'   => 'noindex,follow',
                                   'index,nofollow'   => 'index,nofollow',
                                   'noindex,nofollow' => 'noindex,nofollow',
                      
                               ],
                           ]
                       )
                       ->label(
                           $alias->getAttributeLabel('robots') . ' (' . $languages->getUrlById(
                               $alias->language_id
                           ) . ')'
                       ) ?>
          
          </div>
            
            <?php
        }
    ?>

</div>


<div class="x_panel x_panel-scroll x_panel-blocks">
  
  <div class="style title-edit-page"><?= \Yii::t('catalog', 'Variants') ?></div>
  
  <div class="seo-map-groups-static" id="add-variant-wr" data-number="1">
      
      
      <?php
          foreach ($variants as $key => $variant) {
              $nexKey = $key + 1;
              echo $this->render(
                  '_variant',
                  [
                      'form'    => $form,
                      'variant' => $variant,
                      'key'     => $key,
                      'options' => $options,
                  ]
              );
          }
      ?>
    
    
    <div class="form-group-33-wr">
      <div class="form-group-33" style="height: 421px">
        <a href="#" data-key="<?= $nexKey ?>" id="add-variant" class="add-url-pluse btn btn-success add-gallery-btn-new"><i class="fa fa-plus"></i></a>
      </div>
    </div>
  
  </div>

</div>


<div class="x_panel x_panel-scroll">
  <div class="wrapp-blocks-edit-page">
    <div class="style title-edit-page">Галерея</div>
      <?= GalleryWidget::widget(
          [
              'model' => $product,
          ]
      ); ?>
  </div>
</div>

<div class="style buttons-page-wr">
  <button type="submit" class="btn btn-success">Сохранить</button>
</div>

<?php
    $form::end();
?>
