<?php
    
    namespace artbox\catalog\filter\models;
    
    use artbox\core\models\Language;
    use Yii;
    use yii\db\ActiveQuery;
    use yii\db\ActiveRecord;
    use yii2tech\ar\position\PositionBehavior;
    use yii2tech\ar\variation\VariationBehavior;
    
    /**
     * This is the model class for table "product_option_group_excl".
     *
     * @property integer $id
     * @property boolean $is_filter
     * @property integer $sort
     * @property boolean $status
     * @property integer $type
     * @property integer $created_at
     * @property integer $updated_at
     * @property boolean $in_menu
     * @method GroupLang[] getVariationModels()
     */
    class Group extends ActiveRecord
    {
        /**
         * @inheritdoc
         */
        public function behaviors()
        {
            return [
                'translations'     => [
                    'class'                             => VariationBehavior::class,
                    'variationsRelation'                => 'languages',
                    'defaultVariationRelation'          => 'language',
                    'variationOptionReferenceAttribute' => 'language_id',
                    'optionModelClass'                  => Language::class,
                    'defaultVariationOptionReference'   => function () {
                        return Language::getCurrent()->id;
                    },
                    'optionQueryFilter'                 => function (ActiveQuery $query) {
                        $query->where(
                            [
                                'status' => true,
                            ]
                        );
                    },
                ],
                'positionBehavior' => [
                    'class'             => PositionBehavior::class,
                    'positionAttribute' => 'sort',
                ],
            ];
        }
        
        /**
         * @inheritdoc
         */
        public static function tableName()
        {
            return 'group';
        }
        
        /**
         * @return \yii\db\ActiveQuery
         */
        public function getLanguages()
        {
            return $this->hasMany(GroupLang::class, [ 'group_id' => 'id' ]);
        }
        
        /**
         * @return \yii\db\ActiveQuery
         */
        public function getLanguage()
        {
            return $this->hasOne(GroupLang::class, [ 'group_id' => 'id' ])
                        ->where(
                            [
                                'language_id' => Language::getCurrent()->id,
                            ]
                        );
        }
        
        /**
         * @inheritdoc
         */
        public function rules()
        {
            return [
                [
                    [
                        'sort',
                        'type',
                    ],
                    'integer',
                ],
            ];
        }
        
        /**
         * @inheritdoc
         */
        public function attributeLabels()
        {
            return [
                'id'   => Yii::t('app', 'ID'),
                'sort' => Yii::t('app', 'Sort'),
            ];
        }
    }
