<?php
    
    namespace artbox\catalog\filter\models;
    
    use artbox\core\models\Language;
    use Yii;
    use yii\db\ActiveQuery;
    use yii\db\ActiveRecord;
    use yii2tech\ar\position\PositionBehavior;
    use yii2tech\ar\variation\VariationBehavior;
    
    /**
     * This is the model class for table "product_option_group_excl".
     *
     * @property integer                                    $id
     * @property integer                                    $sort
     * @property boolean                                    $status
     * @property integer                                    $created_at
     * @property integer                                    $updated_at
     * @property integer                                    $group_id
     * @property string                                     $extra
     * @property \artbox\catalog\filter\models\OptionLang   $language
     * @property \artbox\catalog\filter\models\OptionLang[] $languages
     * @method OptionLang getVariationModel( $id )
     * @method OptionLang[] getVariationModels()
     * @method void moveNext()
     * @method void movePrev()
     */
    class Option extends ActiveRecord
    {
        /**
         * @inheritdoc
         */
        public function behaviors()
        {
            return [
                'translations'     => [
                    'class'                             => VariationBehavior::class,
                    'variationsRelation'                => 'languages',
                    'defaultVariationRelation'          => 'language',
                    'variationOptionReferenceAttribute' => 'language_id',
                    'optionModelClass'                  => Language::class,
                    'defaultVariationOptionReference'   => function () {
                        return Language::getCurrent()->id;
                    },
                    'optionQueryFilter'                 => function (ActiveQuery $query) {
                        $query->where(
                            [
                                'status' => true,
                            ]
                        );
                    },
                ],
                'positionBehavior' => [
                    'class'             => PositionBehavior::class,
                    'positionAttribute' => 'sort',
                    'groupAttributes'   => [
                        'group_id',
                    ],
                ],
            ];
        }
        
        /**
         * @inheritdoc
         */
        public static function tableName()
        {
            return 'option';
        }
        
        /**
         * @return \yii\db\ActiveQuery
         */
        public function getLanguages()
        {
            return $this->hasMany(OptionLang::class, [ 'option_id' => 'id' ]);
        }
        
        /**
         * @return \yii\db\ActiveQuery
         */
        public function getLanguage()
        {
            return $this->hasOne(OptionLang::class, [ 'option_id' => 'id' ])
                        ->where(
                            [
                                'language_id' => Language::getCurrent()->id,
                            ]
                        );
        }
        
        /**
         * @inheritdoc
         */
        public function rules()
        {
            return [
                [
                    [
                        'sort',
                        'group_id',
                    ],
                    'integer',
                ],
            ];
        }
        
        /**
         * @inheritdoc
         */
        public function attributeLabels()
        {
            return [
                'id'   => Yii::t('app', 'ID'),
                'sort' => Yii::t('app', 'Sort'),
            ];
        }
        
        public function getGroup()
        {
            return $this->hasOne(Group::class, [ 'id' => 'group_id' ]);
        }
    }
