<?php
    
    namespace artbox\catalog\orders\controllers;
    
    use artbox\core\admin\actions\Create;
    use artbox\core\admin\actions\Delete;
    use artbox\core\admin\actions\Index;
    use artbox\core\admin\actions\Update;
    use artbox\core\admin\actions\View;
    use artbox\core\admin\interfaces\ControllerInterface;
    use artbox\core\admin\widgets\Form;
    use artbox\catalog\orders\models\Delivery;
    use yii\web\Controller;
    use yii\web\NotFoundHttpException;
    
    class DeliveryController extends Controller implements ControllerInterface
    {
        public function actions()
        {
            return [
                'index'  => [
                    'class'            => Index::className(),
                    'columns'          => [
                        'title'  => [
                            'type'         => Index::ACTION_COL,
                            'columnConfig' => [
                                'buttonsTemplate' => '{edit}{delete}',
                            ],
                        ],
                        'sort'   => [
                            'type' => Index::POSITION_COL,
                        ],
                        'status' => [
                            'type' => Index::STATUS_COL,
                        ],
                    ],
                    'model'            => Delivery::className(),
                    'hasLanguage'      => true,
                    'enableMassDelete' => true,
                    'modelPrimaryKey'  => 'id',
                ],
                'create' => array_merge([ 'class' => Create::className() ], self::fieldsConfig()),
                'update' => array_merge([ 'class' => Update::className() ], self::fieldsConfig()),
                'view'   => [
                    'class'          => View::className(),
                    'model'          => Delivery::className(),
                    'hasAlias'       => false,
                    'hasGallery'     => false,
                    'languageFields' => [
                        [
                            'name' => 'title',
                            'type' => Form::STRING,
                        ],
                        [
                            'name' => 'description',
                            'type' => Form::WYSIWYG,
                        ],
                    ],
                    'fields'         => [
                        [
                            'name' => 'status',
                            'type' => Form::BOOL,
                        ],
                        [
                            'name' => 'sort',
                            'type' => Form::NUMBER,
                        ],
                        [
                            'name' => 'value',
                            'type' => Form::NUMBER,
                        ],
                        [
                            'name' => 'color',
                            'type' => Form::STRING,
                        ],
                        [
                            'name' => 'default',
                            'type' => Form::BOOL,
                        ],
                    ],
                ],
                'delete' => [
                    'class' => Delete::className(),
                ],
            ];
        }
        
        public function findModel($id)
        {
            $model = Delivery::find()
                             ->with('languages')
                             ->where([ 'id' => $id ])
                             ->one();
            if ($model !== null) {
                return $model;
            } else {
                throw new NotFoundHttpException('The requested page does not exist.');
            }
        }
        
        public function newModel()
        {
            return new Delivery();
        }
        
        public function deleteModel($id)
        {
            $page = Delivery::find()
                            ->with('languages')
                            ->where(
                                [
                                    'id' => $id,
                                ]
                            )
                            ->one();
            
            return $page->delete();
        }
        
        protected static function fieldsConfig()
        {
            return [
                'model'          => Delivery::className(),
                'hasAlias'       => false,
                'hasGallery'     => false,
                'languageFields' => [
                    [
                        'name' => 'title',
                        'type' => Form::STRING,
                    ],
                    [
                        'name' => 'description',
                        'type' => Form::WYSIWYG,
                    ],
                ],
                'fields'         => [
                    [
                        'name' => 'status',
                        'type' => Form::BOOL,
                    ],
                    [
                        'name' => 'default',
                        'type' => Form::BOOL,
                    ],
                    [
                        'name' => 'sort',
                        'type' => Form::NUMBER,
                    ],
                    [
                        'name' => 'value',
                        'type' => Form::NUMBER,
                    ],
                ],
            ];
        }
    }