<?php
    
    namespace artbox\catalog\orders\controllers;
    
    use artbox\order\models\OrderProductSearch;
    use artbox\catalog\catalog\models\Variant;
    use Yii;
    use artbox\order\models\OrderProduct;
    use yii\helpers\Html;
    use yii\web\Controller;
    use yii\web\NotFoundHttpException;
    use yii\filters\VerbFilter;
    use yii\filters\AccessControl;
    use yii\web\Response;
    
    /**
     * OrderProductController implements the CRUD actions for OrderProduct model.
     */
    class OrderProductController extends Controller
    {
        /**
         * @inheritdoc
         */
        public function behaviors()
        {
            return [
                'verbs'  => [
                    'class'   => VerbFilter::className(),
                    'actions' => [
                        'delete' => [ 'POST' ],
                    ],
                ],
                'access' => [
                    'class' => AccessControl::className(),
                    'rules' => [
                        [
                            'actions' => [
                                'login',
                                'error',
                            ],
                            'allow'   => true,
                        ],
                        [
                            'allow' => true,
                            'roles' => [ '@' ],
                        ],
                    ],
                ],
            ];
        }
        
        /**
         * Lists all OrderProduct models.
         *
         * @return mixed
         */
        public function actionIndex()
        {
            $searchModel = new OrderProductSearch();
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
            
            return $this->render(
                '@artbox/order/views/order-product/index',
                [
                    'searchModel'  => $searchModel,
                    'dataProvider' => $dataProvider,
                ]
            );
        }
        
        /**
         * Displays a single OrderProduct model.
         *
         * @param integer $id
         *
         * @return mixed
         * @throws \yii\web\NotFoundHttpException
         */
        public function actionView($id)
        {
            return $this->render(
                '@artbox/order/views/order-product/view',
                [
                    'model' => $this->findModel($id),
                ]
            );
        }
        
        /**
         * Creates a new OrderProduct model.
         * If creation is successful, the browser will be redirected to the 'view' page.
         *
         * @return mixed
         */
        public function actionCreate()
        {
            $model = new OrderProduct();
            
            if ($model->load(Yii::$app->request->post()) && $model->save()) {
                return $this->redirect(
                    [
                        'view',
                        'id' => $model->id,
                    ]
                );
            } else {
                return $this->render(
                    '@artbox/order/views/order-product/create',
                    [
                        'model' => $model,
                    ]
                );
            }
        }
        
        /**
         * Updates an existing OrderProduct model.
         * If update is successful, the browser will be redirected to the 'view' page.
         *
         * @param integer $id
         *
         * @return mixed
         * @throws \yii\web\NotFoundHttpException
         */
        public function actionUpdate($id)
        {
            $model = $this->findModel($id);
            
            if ($model->load(Yii::$app->request->post()) && $model->save()) {
                return $this->redirect(
                    [
                        'view',
                        'id' => $model->id,
                    ]
                );
            } else {
                return $this->render(
                    '@artbox/order/views/order-product/update',
                    [
                        'model' => $model,
                    ]
                );
            }
        }
        
        /**
         * Deletes an existing OrderProduct model.
         * If deletion is successful, the browser will be redirected to the 'index' page.
         *
         * @param integer $id
         *
         * @return mixed
         * @throws \Throwable
         * @throws \yii\db\StaleObjectException
         * @throws \yii\web\NotFoundHttpException
         */
        public function actionDelete($id)
        {
            $this->findModel($id)
                 ->delete();
            
            return $this->redirect([ 'index' ]);
        }
        
        /**
         * Finds the OrderProduct model based on its primary key value.
         * If the model is not found, a 404 HTTP exception will be thrown.
         *
         * @param integer $id
         *
         * @return OrderProduct the loaded model
         * @throws NotFoundHttpException if the model cannot be found
         */
        protected function findModel($id)
        {
            if (( $model = OrderProduct::findOne($id) ) !== null) {
                return $model;
            } else {
                throw new NotFoundHttpException('The requested page does not exist.');
            }
        }
        
        /**
         * @param $variant_id
         * @param $count
         *
         * @return array
         */
        public function actionAdd($variant_id, $count)
        {
            \Yii::$app->response->format = Response::FORMAT_JSON;
            
            /**
             * @var Variant $variant
             */
            $variant = Variant::find()
                              ->where([ 'id' => $variant_id ])
                              ->with('product.lang')
                              ->one();
            
            if ($variant) {
                $result = '<div id="' . $variant->id . '" class="row"><div class="col-md-8">';
                $result .= $variant->product->lang->title . ' (' . $variant->sku . ')';
                $result .= '</div><div class="col-md-3">';
                $result .= Html::input(
                    'number',
                    'Product[' . $variant->id . ']',
                    empty($count) ? 1 : $count,
                    [
                        'class' => 'form-control count-input',
                        'min'   => 1,
                    ]
                );
                $result .= '</div><div class="col-md-1">';
                $result .= Html::a(
                    '<i class="fa fa-trash"></i>',
                    '#',
                    [
                        'class' => 'btn btn-primary delete-product-row',
                    ]
                );
                $result .= '</div></div>';
            } else {
                $result = '';
            }
            
            return [
                'row'   => $result,
                'id'    => empty($variant) ? 0 : $variant->id,
                'count' => empty($count) ? 1 : intval($count),
            ];
        }
    }
